/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package bia.moea;

/**
 * @author Jiri Kubalik
 */
public class KnapsackIndividual{

    public static double[] capacities = new double[2];      //--- capacities of the knapsacks
    public static KnapsackItem[] items;            //--- definitions of items
    public static int[][] WeightProfitOrder; //--- list of items indexes ordered by weight/profit ratio from the largest value to the smallest one    
    
    /**
     * Genes of individual binary represented .
     */
    public int[] genes;

    /**
     * Objective values of the individual.
     */
    public int[] objectives;

    /**
     * Fitness value of the individual.
     */
    public double fitness;
    
    /**
     * Number of the non-dominated front the individual belongs to. 
     * The lower value, the better front.
     */
    public int frontId; 

    /**
     * Crowding distance -- the size of the individual's neighborhood.
     * The higher value the better.
     */
    public int crowdingDist; 

    /**
     * NSGA parameters
     */
    public static KnapsackParameters parameters = KnapsackParameters.getInstance();

    /**
     * Constructor. Generates random individual.
     */
    public KnapsackIndividual() {

        objectives = new int[parameters.getNbObjectives()];
        frontId = -1;
        crowdingDist = 1;
        
        // Allocate memory for genes array.
        this.genes = new int[parameters.getNbItems()];
        // Randomly initialise genes of individual.
        for (int genIter = 0; genIter < genes.length; genIter++) {
            if (Math.random() < 0.5) this.genes[genIter] = 0;
            else this.genes[genIter] = 1;
        }
    }

    /**
     * Constructor generating empty individual without its genes defined.
     *
     * @param b dump parameter.
     */
    public KnapsackIndividual(boolean b) {
        
        objectives = new int[parameters.getNbObjectives()];
        frontId = -1;
        crowdingDist = 1;
        
        // Allocate memory for genes array.
        this.genes = new int[parameters.getNbItems()];
    }

    /**
     * Cloning method for creating deep copies of individual instances.
     *
     * @return
     */
    @Override
    public KnapsackIndividual clone() {
        KnapsackIndividual ind = new KnapsackIndividual(false); // Create empty idividual.

        //--- Copy genes
        ind.genes = new int[parameters.getNbItems()];
        for (int geneIter = 0; geneIter < parameters.getNbItems(); geneIter++) {
            ind.genes[geneIter] = this.genes[geneIter];
        }

        //--- Copy objectives
        ind.objectives = new int[parameters.getNbObjectives()];
        for (int i = 0; i < parameters.getNbObjectives(); i++) {
            ind.objectives[i] = this.objectives[i];
        }

        ind.frontId = this.frontId;
        ind.crowdingDist = this.crowdingDist;
        ind.fitness = this.fitness;
        
        return ind;
    }

    /**
     * Objective values calculation.
     */
    public void getObjectives() {
        int[] totalWeight = {0,0};  //--- weights and profits of the whole knapsacks
        int[] totalProfit = {0,0};

        //--- Check the solution and correct it if needed.
        for(int j = 0; j < parameters.getNbItems(); j++){
            if(genes[j] == 1){
                totalWeight[0] += items[j].getWeight(0);
                totalWeight[1] += items[j].getWeight(1);
                //---
                totalProfit[0] += items[j].getProfit(0);
                totalProfit[1] += items[j].getProfit(1);
            }
        }
        int j0 = 0;
        int j1 = 0;
        while((totalWeight[0] > capacities[0]) || (totalWeight[1] > capacities[1])){
            if(Math.random() < 0.5){
                //--- Correct w.r.t. the first knapsack WeightProfitOrder
                while(genes[WeightProfitOrder[0][j0]] == 0)
                    j0++;
                //--- Remove items starting from the least valuable to the most valuable one.
                //--- NSGA2Knapsack.WeightProfitOrder[i][j] is index of the current least valuable item.
                genes[WeightProfitOrder[0][j0]] = 0;
                totalWeight[0] -= items[WeightProfitOrder[0][j0]].getWeight(0);
                totalProfit[0] -= items[WeightProfitOrder[0][j0]].getProfit(0);
                totalWeight[1] -= items[WeightProfitOrder[0][j0]].getWeight(1);
                totalProfit[1] -= items[WeightProfitOrder[0][j0]].getProfit(1);
                j0++;
            }
            else{
                //--- Correct w.r.t. the second knapsack WeightProfitOrder
                while(genes[WeightProfitOrder[1][j1]] == 0)
                    j1++;
                //--- Remove items starting from the least valuable to the most valuable one.
                //--- NSGA2Knapsack.WeightProfitOrder[i][j] is index of the current least valuable item.
                genes[WeightProfitOrder[1][j1]] = 0;
                totalWeight[1] -= items[WeightProfitOrder[1][j1]].getWeight(1);
                totalProfit[1] -= items[WeightProfitOrder[1][j1]].getProfit(1);
                totalWeight[0] -= items[WeightProfitOrder[1][j1]].getWeight(0);
                totalProfit[0] -= items[WeightProfitOrder[1][j1]].getProfit(0);
                j1++;
            }
        }
        
        //--- Assign objectives, which are the totalProfit values
        this.objectives[0] = totalProfit[0];
        this.objectives[1] = totalProfit[1];
    }
    
    /**
     * Fitness value calculation.
     */
    public void getFitness() {
        
        //--- Calculate objectives
        getObjectives();
        
        //--- Calculate single fitness value as the weighted sum of its objectives
        fitness = parameters.getAlpha() * objectives[0] + (1.0 - parameters.getAlpha()) * objectives[1];
    }

    /**
     * Sets the crowding distance to @param cd.
     */
    public void setCrowdingDist(int cd){
        this.crowdingDist = cd;
    }

    /**
     * Checks whether individual this dominates individual @param ind. 
     * @return true if this dominates @param ind, otherwise returns false.
     */
    public boolean dominates(KnapsackIndividual ind){
    /* PUT YOUR CODE HERE */
 
        //--- 2 maximization objectives are considered
        if(((this.objectives[0] >= ind.objectives[0]) && (this.objectives[1] > ind.objectives[1])) ||
           ((this.objectives[0] > ind.objectives[0]) && (this.objectives[1] >= ind.objectives[1])))
            return true;
        else
            return false;
    }
    
    /**
     * One-point crossover.
     *
     * @param mate second individual in crossover
     * @return newly generated individuals
     */
    public KnapsackIndividual[] one_point_crossover(KnapsackIndividual mate) {
        KnapsackIndividual[] I = new KnapsackIndividual[2];
        // Make clones of this and input individual.
        I[0] = this.clone();
        I[1] = mate.clone();

        // Generate crosspoint.
        int cross_point = (int) (Math.random() * (parameters.getNbItems() - 1));

        // Crossing method.
        // Switch first part of chromozomes.
        for (int i = 0; i <= cross_point; i++) {
            I[0].genes[i] = mate.genes[i];
            I[1].genes[i] = this.genes[i];
        }
        // Copy rest of chromozomes.
        for (int i = cross_point + 1; i < parameters.getNbItems(); i++) {
            I[0].genes[i] = this.genes[i];
            I[1].genes[i] = mate.genes[i];
        }

        return I;
    }

    /**
     * Two-point crossover.
     *
     * @param mate second individual in crossover
     * @return newly generated individuals
     */
    public KnapsackIndividual[] two_point_crossover(KnapsackIndividual mate) {
        KnapsackIndividual[] I = new KnapsackIndividual[2];
        // Make clones of this and input individual.
        I[0] = this.clone();
        I[1] = mate.clone();

        // Generate crosspoints.
        int cross_point1 = (int) (Math.random() * parameters.getNbItems());
        int cross_point2 = cross_point1;
        // While both cross points are the same, generate new cross point.
        while (cross_point1 == cross_point2) {
            cross_point2 = (int) (Math.random() * parameters.getNbItems());
        }
        // Switch both cross point so second is larger than first one.
        if (cross_point2 < cross_point1) {
            cross_point1 = cross_point1 + cross_point2;
            cross_point2 = cross_point1 - cross_point2;
            cross_point1 = cross_point1 - cross_point2;
        }

        // Crossing method.
        // Copy first part of chromozomes.
        for (int i = 0; i <= cross_point1; i++) {
            I[0].genes[i] = this.genes[i];
            I[1].genes[i] = mate.genes[i];
        }
        // Switch middle parts of chromozomes.
        for (int i = cross_point1 + 1; i <= cross_point2; i++) {
            I[0].genes[i] = mate.genes[i];
            I[1].genes[i] = this.genes[i];
        }
        // Copy rest of chromozomes.
        for (int i = cross_point2 + 1; i < parameters.getNbItems(); i++) {
            I[0].genes[i] = this.genes[i];
            I[1].genes[i] = mate.genes[i];
        }

        return I;
    }

    /**
     * Uniform crossover. Each gene is switched under some probability value.
     *
     * @param mate second individual in crossover
     * @return newly generated individuals
     */
    public KnapsackIndividual[] uniform_crossover(KnapsackIndividual mate) {

        KnapsackIndividual[] I = new KnapsackIndividual[2];
        // Make clones of this and input individual.
        I[0] = this.clone();
        I[1] = mate.clone();

        // Each gene is inherited either from the 1st or the 2nd parent
        for (int geneIter = 0; geneIter < parameters.getNbItems(); geneIter++) {
            if (Math.random() < 0.5)
                I[0].genes[geneIter] = mate.genes[geneIter];
            else
                I[0].genes[geneIter] = this.genes[geneIter];

            if (Math.random() < 0.5)
                I[1].genes[geneIter] = this.genes[geneIter];
            else
                I[1].genes[geneIter] = mate.genes[geneIter];
        }

        return I;
    }

    /**
     * Find proper crossover according settings in SGA.java file.
     *
     * @param mate second individual in crossover
     * @return newly generated individuals
     */
    public KnapsackIndividual[] crossover(KnapsackIndividual mate) {
        switch (parameters.getCrossType()) {
            case 0:
                return uniform_crossover(mate);
            case 1:
                return one_point_crossover(mate);
            case 2:
                return two_point_crossover(mate);
            default:
                System.err.println("ERROR - Bad value of crossType variable. Check if its value is between 0 and 2.");
                System.exit(1);
                return null;
        }
    }
    
    /**
     * Simple bit-flip mutation.
     */
    public void mutation() {

        for (int geneIter = 0; geneIter < parameters.getNbItems(); geneIter++) {
            // Mutate each gene with probability Pm
            if (Math.random() < parameters.getPM()) {
                this.genes[geneIter] = (this.genes[geneIter] + 1) % 2;
            }
        }
    }
}
