/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package bia.moea;

import static bia.moea.NSGA2Knapsack.dataset;
import bia.tsp.Node;
import java.util.ArrayList;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JFrame;
import java.io.*;

/**
 *
 * @author Jiri Kubalik
 */
public class KnapsackViz extends Canvas {

		  public static Node[] initialFrontNodes;
		  public static Node[] finalFrontNodes;
		  public static ArrayList<Node> paretoFrontNodes = null;
		  public static double maxX = Double.MIN_VALUE;	  // Maximal X-posityion from all cities
		  public static double minX = Double.MAX_VALUE;	  // Minimal X-posityion from all cities
		  public static double maxY = Double.MIN_VALUE;	  // Maximal Y-posityion from all cities
		  public static double minY = Double.MAX_VALUE;	  // Minimal Y-posityion from all cities
		  public static double multCoefX;				// Just coeficient to fit instance into window.
		  public static double multCoefY;				// Just coeficient to fit instance into window.
		  public static int frameSize = 600;		// Size of graphic window.
		  public static int circleSize = 4;			// Size of the circle makr for cities.

		  public KnapsackViz() {

					 setSize(frameSize, frameSize);
					 setBackground(Color.WHITE);
		  }
          
		  public void paint(Graphics g) {
              int offset = 10;

                Graphics2D g2d = (Graphics2D) g;

                //--- Draw initialFrontNodes with red color.
                //--- Constructor for Ellipse2D is X,Y,W,H and X,Y is upper corner of bounding box for Ellipse2D, 
                //--- so there shoud be that half of its diameter (W,H) substracted from X and Y
                g2d.setColor(Color.red);
                for (int i = 0; i < initialFrontNodes.length; i++) {
                    // Adjust coordinates to window.
                    double x = offset + (initialFrontNodes[i].x-minX) * multCoefX;
                    double y = offset + (initialFrontNodes[i].y-minY) * multCoefY;
                    // Again bounding box for ellipse.
                    Ellipse2D.Double city = new Ellipse2D.Double(x-circleSize, y-circleSize, 2*circleSize, 2*circleSize);
                    g2d.fill(city);
                }

                //--- Draw finalFrontNodes with blue color.
                g2d.setColor(Color.blue);
                for (int i = 0; i < finalFrontNodes.length; i++) {
                    // Adjust coordinates to window.
                    double x = offset + (finalFrontNodes[i].x-minX) * multCoefX;
                    double y = offset + (finalFrontNodes[i].y-minY) * multCoefY;
                    // Again bounding box for ellipse.
                    Ellipse2D.Double city = new Ellipse2D.Double(x-circleSize, y-circleSize, 2*circleSize, 2*circleSize);
                    g2d.fill(city);
                }
                
                //--- Draw Pareto-optimal frony with black color
                if (paretoFrontNodes != null){
                    g2d.setColor(Color.black);
                    for (Node node : paretoFrontNodes) {
                        // Adjust coordinates to window.
                        double x = offset + (node.x-minX) * multCoefX;
                        double y = offset + (node.y-minY) * multCoefY;
                        // Again bounding box for ellipse.
                        Ellipse2D.Double city = new Ellipse2D.Double(x-circleSize, y-circleSize, 2*circleSize, 2*circleSize);
                        g2d.fill(city);
                    }
                }
          }

    /**
     * Visualizes the initial and final non-dominated front and the Pareto-optimal front.
     * 
     */
    public static void visualizeNonDominatedFronts(ArrayList<KnapsackIndividual> initialFront, ArrayList<KnapsackIndividual> finalFront) {

        KnapsackViz knapsackVisualizator = new KnapsackViz(); 
        JFrame aFrame = null;

        //--- Load Pareto-optimal front
        KnapsackParameters parameters = KnapsackParameters.getInstance();
        File dataset = new File(parameters.getDataset() + ".pareto");
        if (dataset.exists()) {
            try {
                BufferedReader br = new BufferedReader(new FileReader(dataset));
                String temp;
                KnapsackViz.paretoFrontNodes = new ArrayList<Node>();
                //---
                temp = br.readLine();
                while(temp != null){
                    String[] line = temp.split("\\s++");
                    Node newNode = new Node(Integer.parseInt(line[0]), Integer.parseInt(line[1]));
                    if(newNode.x < KnapsackViz.minX) KnapsackViz.minX = newNode.x;
                    if(newNode.x > KnapsackViz.maxX) KnapsackViz.maxX = newNode.x;
                    if(newNode.y < KnapsackViz.minY) KnapsackViz.minY = newNode.y;
                    if(newNode.y > KnapsackViz.maxY) KnapsackViz.maxY = newNode.y;
                    KnapsackViz.paretoFrontNodes.add(newNode);
                    temp = br.readLine();
                }
                br.close();
            } catch (IOException e) {
                System.out.println(e.getMessage());
            }
        }
        
        //--- Visualize the initialFront
        KnapsackViz.initialFrontNodes = new Node[initialFront.size()];
        int i = 0;
        for (KnapsackIndividual ind : initialFront) {
            KnapsackViz.initialFrontNodes[i] = new Node(ind.objectives[0], ind.objectives[1]);
            if(KnapsackViz.initialFrontNodes[i].x < KnapsackViz.minX) KnapsackViz.minX = KnapsackViz.initialFrontNodes[i].x;
            if(KnapsackViz.initialFrontNodes[i].x > KnapsackViz.maxX) KnapsackViz.maxX = KnapsackViz.initialFrontNodes[i].x;
            if(KnapsackViz.initialFrontNodes[i].y < KnapsackViz.minY) KnapsackViz.minY = KnapsackViz.initialFrontNodes[i].y;
            if(KnapsackViz.initialFrontNodes[i].y > KnapsackViz.maxY) KnapsackViz.maxY = KnapsackViz.initialFrontNodes[i].y;
            i++;
        }

        //--- Visualize the finalFront
        KnapsackViz.finalFrontNodes = new Node[finalFront.size()];
        i = 0;
        for (KnapsackIndividual ind : finalFront) {
            KnapsackViz.finalFrontNodes[i] = new Node(ind.objectives[0], ind.objectives[1]);
            if(KnapsackViz.finalFrontNodes[i].x < KnapsackViz.minX) KnapsackViz.minX = KnapsackViz.finalFrontNodes[i].x;
            if(KnapsackViz.finalFrontNodes[i].x > KnapsackViz.maxX) KnapsackViz.maxX = KnapsackViz.finalFrontNodes[i].x;
            if(KnapsackViz.finalFrontNodes[i].y < KnapsackViz.minY) KnapsackViz.minY = KnapsackViz.finalFrontNodes[i].y;
            if(KnapsackViz.finalFrontNodes[i].y > KnapsackViz.maxY) KnapsackViz.maxY = KnapsackViz.finalFrontNodes[i].y;
            i++;
        }

        // Get coeficient for better drawing.
        KnapsackViz.multCoefX = (KnapsackViz.frameSize - 40) / Math.abs(KnapsackViz.maxX - KnapsackViz.minX);
        KnapsackViz.multCoefY = (KnapsackViz.frameSize - 60) / Math.abs(KnapsackViz.maxY - KnapsackViz.minY);

        //create a new frame to which we will add a canvas
        aFrame = new JFrame();
        //aFrame.setAction(JFrame.EXIT_ON_CLOSE);
        aFrame.setSize(KnapsackViz.frameSize, KnapsackViz.frameSize);
        aFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //add the canvas
        aFrame.add(knapsackVisualizator);
        aFrame.setTitle("Non-dominated fronts");

        aFrame.setVisible(true);
    }
}
